﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Globalization;
using System.Collections.ObjectModel;
using System.Reflection;
using C1.WPF.DataGrid;
using System.Windows.Media.Animation;

namespace DataGridCellAnimation
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        public MainWindow()
        {
            InitializeComponent();
            datagrid.ItemsSource = FinancialData.GetFinancialData();
            datagrid.LoadedCellPresenter += new EventHandler<C1.WPF.DataGrid.DataGridCellEventArgs>(datagrid_LoadedCellPresenter);
            datagrid.UnloadedCellPresenter += new EventHandler<DataGridCellEventArgs>(datagrid_UnloadedCellPresenter);
        }

        void datagrid_UnloadedCellPresenter(object sender, DataGridCellEventArgs e)
        {
            if ((e.Cell.Column.Index == 3) || (e.Cell.Column.Index == 4))
            {
                e.Cell.Presenter.Triggers.Clear();
            }
        }

        void datagrid_LoadedCellPresenter(object sender, C1.WPF.DataGrid.DataGridCellEventArgs e)
        {
            if ((e.Cell.Column.Index == 3) || (e.Cell.Column.Index == 4))
            {
                e.Cell.Presenter.DataContext = e.Cell.Row.DataItem;
                TextBlock tb = e.Cell.Presenter.Content as TextBlock;

                Binding bcurrent = tb.GetBindingExpression(TextBlock.TextProperty).ParentBinding;
                Binding bnew = CopyBinding(ref bcurrent);
                tb.SetBinding(TextBlock.TextProperty, bnew);

                Dispatcher.BeginInvoke(new System.Threading.ThreadStart(() =>
                    {
                        var ca = new ColorAnimation(Colors.Transparent, Colors.Red, TimeSpan.FromSeconds(1));
                        Storyboard.SetTarget(ca, e.Cell.Presenter);
                        Storyboard.SetTargetProperty(ca, new PropertyPath("(Background).(SolidColorBrush.Color)"));
                        
                        var et = new EventTrigger(Binding.TargetUpdatedEvent);
                        et.Actions.Add(new BeginStoryboard
                        {
                            Storyboard = new Storyboard
                            {
                                Duration = TimeSpan.FromSeconds(1),
                                Children = { ca },
                                AutoReverse = true,
                            }
                        });
                        e.Cell.Presenter.Triggers.Add(et);
                    }), System.Windows.Threading.DispatcherPriority.Loaded);
            }
        }

        private Binding CopyBinding(ref Binding CurrentBinding)
        {
            Binding NewBinding = new Binding();
            PropertyInfo[] fields = typeof(Binding).GetProperties();
            foreach (PropertyInfo pi in fields)
            {
                if ((pi.GetValue(CurrentBinding, null) != null) && (pi.CanWrite == true))
                    pi.SetValue(NewBinding, pi.GetValue(CurrentBinding, null), null);
            }
            NewBinding.NotifyOnTargetUpdated = true;
            return NewBinding;
        }
    }


}